@@ -57,10 +57,11 @@ function show_options {
     echo "      --overcloud-images   -- Build and load overcloud images."
     echo "      --register-nodes     -- Register and configure nodes."
     echo "      --introspect-nodes   -- Introspect nodes."
-    echo "      --flavors            -- Create flavors for deployment."
     echo "      --overcloud-deploy   -- Deploy an overcloud."
+    echo "      --overcloud-update   -- Update a deployed overcloud."
     echo "      --overcloud-delete   -- Delete the overcloud."
     echo "      --use-containers     -- Use a containerized compute node."
+    echo "      --enable-check       -- Enable checks on update."
     echo "      --overcloud-pingtest -- Run a tenant vm, attach and ping floating IP."
     echo "      --all, -a            -- Run all of the above commands."
     echo "      -x                   -- enable tracing"
@@ -75,7 +76,7 @@ if [ ${#@} = 0 ]; then
 fi
 
 TEMP=$(getopt -o ,h \
-        -l,help,repo-setup,delorean-setup,delorean-build,undercloud,overcloud-images,register-nodes,introspect-nodes,flavors,overcloud-deploy,overcloud-delete,use-containers,overcloud-pingtest,all \
+        -l,help,repo-setup,delorean-setup,delorean-build,undercloud,overcloud-images,register-nodes,introspect-nodes,overcloud-deploy,overcloud-update,overcloud-delete,use-containers,overcloud-pingtest,all,enable-check \
         -o,x,h,a \
         -n $SCRIPT_NAME -- "$@")
 
@@ -88,12 +89,11 @@ fi
 eval set -- "$TEMP"
 
 ALL=${ALL:-""}
-STABLE_RELEASE=${STABLE_RELEASE:-}
 CONTAINER_ARGS=${CONTAINER_ARGS:-"-e /usr/share/openstack-tripleo-heat-templates/environments/docker.yaml -e /usr/share/openstack-tripleo-heat-templates/environments/docker-network.yaml --libvirt-type=qemu"}
+STABLE_RELEASE=${STABLE_RELEASE:-}
 DELOREAN_REPO_FILE=${DELOREAN_REPO_FILE:-"delorean.repo"}
 DELOREAN_REPO_URL=${DELOREAN_REPO_URL:-"\
     http://trunk.rdoproject.org/centos7/current-tripleo/"}
-FLAVORS=${FLAVORS:-""}
 DELOREAN_STABLE_REPO_URL=${DELOREAN_STABLE_REPO_URL:-"\
     https://trunk.rdoproject.org/centos7-$STABLE_RELEASE/current/"}
 ATOMIC_URL=${ATOMIC_URL:-"https://download.fedoraproject.org/pub/fedora/linux/releases/22/Cloud/x86_64/Images/Fedora-Cloud-Atomic-22-20150521.x86_64.qcow2"}
@@ -104,15 +104,22 @@ OVERCLOUD_DEPLOY=${OVERCLOUD_DEPLOY:-""}
 OVERCLOUD_DELETE=${OVERCLOUD_DELETE:-""}
 OVERCLOUD_DELETE_TIMEOUT=${OVERCLOUD_DELETE_TIMEOUT:-"300"}
 OVERCLOUD_DEPLOY_ARGS=${OVERCLOUD_DEPLOY_ARGS:-""}
+OVERCLOUD_UPDATE=${OVERCLOUD_UPDATE:-""}
+OVERCLOUD_UPDATE_RM_FILES=${OVERCLOUD_UPDATE_RM_FILES:-"1"}
+OVERCLOUD_UPDATE_ARGS=${OVERCLOUD_UPDATE_ARGS:-"$OVERCLOUD_DEPLOY_ARGS"}
+OVERCLOUD_UPDATE_CHECK=${OVERCLOUD_UPDATE_CHECK:-}
 OVERCLOUD_IMAGES_PATH=${OVERCLOUD_IMAGES_PATH:-"$HOME"}
 OVERCLOUD_IMAGES=${OVERCLOUD_IMAGES:-""}
-OVERCLOUD_IMAGES_DIB_YUM_REPO_CONF=${OVERCLOUD_IMAGES_DIB_YUM_REPO_CONF:-"\
-    /etc/yum.repos.d/delorean.repo \
-    /etc/yum.repos.d/delorean-current.repo \
-    /etc/yum.repos.d/delorean-deps.repo"}
+OVERCLOUD_IMAGES_ARGS=${OVERCLOUD_IMAGES_ARGS='--all'}
 OVERCLOUD_NAME=${OVERCLOUD_NAME:-"overcloud"}
 OVERCLOUD_PINGTEST=${OVERCLOUD_PINGTEST:-""}
+OVERCLOUD_PINGTEST_OLD_HEATCLIENT=${OVERCLOUD_PINGTEST_OLD_HEATCLIENT:-"1"}
 REPO_SETUP=${REPO_SETUP:-""}
+REPO_PREFIX=${REPO_PREFIX:-"/etc/yum.repos.d/"}
+OVERCLOUD_IMAGES_DIB_YUM_REPO_CONF=${OVERCLOUD_IMAGES_DIB_YUM_REPO_CONF:-"\
+    $REPO_PREFIX/delorean.repo \
+    $REPO_PREFIX/delorean-current.repo \
+    $REPO_PREFIX/delorean-deps.repo"}
 DELOREAN_SETUP=${DELOREAN_SETUP:-""}
 DELOREAN_BUILD=${DELOREAN_BUILD:-""}
 STDERR=/dev/null
@@ -124,14 +131,17 @@ USE_CONTAINERS=${USE_CONTAINERS:-""}
 # TODO: remove this when Image create in openstackclient supports the v2 API
 export OS_IMAGE_API_VERSION=1
 
+# Temporary workarounds
+
 while true ; do
     case "$1" in
         --all|-a ) ALL="1"; shift 1;;
         --use-containers) USE_CONTAINERS="1"; shift 1;;
-        --flavors) FLAVORS="1"; shift 1;;
+        --enable-check) OVERCLOUD_UPDATE_CHECK="1"; shift 1;;
         --introspect-nodes) INTROSPECT_NODES="1"; shift 1;;
         --register-nodes) REGISTER_NODES="1"; shift 1;;
         --overcloud-deploy) OVERCLOUD_DEPLOY="1"; shift 1;;
+        --overcloud-update) OVERCLOUD_UPDATE="1"; shift 1;;
         --overcloud-delete) OVERCLOUD_DELETE="1"; shift 1;;
         --overcloud-images) OVERCLOUD_IMAGES="1"; shift 1;;
         --overcloud-pingtest) OVERCLOUD_PINGTEST="1"; shift 1;;
@@ -184,32 +194,32 @@ function repo_setup {
 
     if [ -z "$STABLE_RELEASE" ]; then
         # Enable the Delorean Deps repository
-        sudo curl -o /etc/yum.repos.d/delorean-deps.repo http://trunk.rdoproject.org/centos7/delorean-deps.repo
-        sudo sed -i -e 's%priority=.*%priority=30%' /etc/yum.repos.d/delorean-deps.repo
+        sudo curl -o $REPO_PREFIX/delorean-deps.repo http://trunk.rdoproject.org/centos7/delorean-deps.repo
+        sudo sed -i -e 's%priority=.*%priority=30%' $REPO_PREFIX/delorean-deps.repo
 
         # Enable last known good RDO Trunk Delorean repository
-        sudo curl -o /etc/yum.repos.d/delorean.repo $DELOREAN_REPO_URL/$DELOREAN_REPO_FILE
-        sudo sed -i -e 's%priority=.*%priority=20%' /etc/yum.repos.d/delorean.repo
+        sudo curl -o $REPO_PREFIX/delorean.repo $DELOREAN_REPO_URL/$DELOREAN_REPO_FILE
+        sudo sed -i -e 's%priority=.*%priority=20%' $REPO_PREFIX/delorean.repo
 
         # Enable latest RDO Trunk Delorean repository
-        sudo curl -o /etc/yum.repos.d/delorean-current.repo http://trunk.rdoproject.org/centos7/current/delorean.repo
-        sudo sed -i -e 's%priority=.*%priority=10%' /etc/yum.repos.d/delorean-current.repo
-        sudo sed -i 's/\[delorean\]/\[delorean-current\]/' /etc/yum.repos.d/delorean-current.repo
-        sudo /bin/bash -c "cat <<-EOF>>/etc/yum.repos.d/delorean-current.repo
+        sudo curl -o $REPO_PREFIX/delorean-current.repo http://trunk.rdoproject.org/centos7/current/delorean.repo
+        sudo sed -i -e 's%priority=.*%priority=10%' $REPO_PREFIX/delorean-current.repo
+        sudo sed -i 's/\[delorean\]/\[delorean-current\]/' $REPO_PREFIX/delorean-current.repo
+        sudo /bin/bash -c "cat <<-EOF>>$REPO_PREFIX/delorean-current.repo
 
 includepkgs=diskimage-builder,instack,instack-undercloud,os-apply-config,os-cloud-config,os-collect-config,os-net-config,os-refresh-config,python-tripleoclient,tripleo-common,openstack-tripleo-heat-templates,openstack-tripleo-image-elements,openstack-tripleo,openstack-tripleo-puppet-elements
 EOF"
     else
         # Enable the Delorean Deps repository
-        sudo curl -o /etc/yum.repos.d/delorean-deps.repo http://trunk.rdoproject.org/centos7-$STABLE_RELEASE/delorean-deps.repo
-        sudo sed -i -e 's%priority=.*%priority=30%' /etc/yum.repos.d/delorean-deps.repo
+        sudo curl -o $REPO_PREFIX/delorean-deps.repo http://trunk.rdoproject.org/centos7-$STABLE_RELEASE/delorean-deps.repo
+        sudo sed -i -e 's%priority=.*%priority=30%' $REPO_PREFIX/delorean-deps.repo
 
         # Enable delorean current for the stable version
-        sudo curl -o /etc/yum.repos.d/delorean.repo $DELOREAN_STABLE_REPO_URL/$DELOREAN_REPO_FILE
+        sudo curl -o $REPO_PREFIX/delorean.repo $DELOREAN_STABLE_REPO_URL/$DELOREAN_REPO_FILE
         sudo sed -i -e 's%priority=.*%priority=20%' /etc/yum.repos.d/delorean.repo
 
         # Create empty delorean-current for dib image building
-        sudo sh -c '> /etc/yum.repos.d/delorean-current.repo'
+        sudo sh -c "> $REPO_PREFIX/delorean-current.repo"
     fi
 
     # Install the yum-plugin-priorities package so that the Delorean repository
@@ -369,7 +379,7 @@ function overcloud_images {
     log "Overcloud images saved in $OVERCLOUD_IMAGES_PATH"
     pushd $OVERCLOUD_IMAGES_PATH
     DIB_YUM_REPO_CONF=$OVERCLOUD_IMAGES_DIB_YUM_REPO_CONF \
-        openstack overcloud image build --all 2>&1 | \
+        openstack overcloud image build $OVERCLOUD_IMAGES_ARGS 2>&1 | \
         tee -a overcloud-image-build.log
 
     stackrc_check
@@ -410,21 +420,6 @@ function introspect_nodes {
 
 }
 
-
-function flavors {
-
-    log "Flavors"
-
-    stackrc_check
-    openstack flavor create --id auto --ram 4096 --disk 40 --vcpus 1 baremetal
-    openstack flavor \
-        set --property "capabilities:boot_option"="local" \
-        baremetal
-
-    log "Flavors - DONE."
-
-}
-
 function overcloud_deploy {
 
     log "Overcloud deploy"
@@ -443,9 +438,46 @@ function overcloud_deploy {
         OVERCLOUD_DEPLOY_ARGS="$OVERCLOUD_DEPLOY_ARGS $CONTAINER_ARGS"
     fi
 
+    log "Overcloud create started."
     openstack overcloud deploy $OVERCLOUD_DEPLOY_ARGS
-    log "Overcloud deployment started - DONE."
+    log "Overcloud create - DONE."
+}
 
+function overcloud_update {
+    # Force use of --templates
+    if [[ ! $OVERCLOUD_UPDATE_ARGS =~ --templates ]]; then
+        OVERCLOUD_UPDATE_ARGS="$OVERCLOUD_UPDATE_ARGS --templates"
+    fi
+    stackrc_check
+    if heat stack-show "$OVERCLOUD_NAME" | grep "stack_status " | egrep "(CREATE|UPDATE)_COMPLETE"; then
+        FILE_PREFIX=$(date "+overcloud-update-resources-%s")
+        BEFORE_FILE="/tmp/${FILE_PREFIX}-before.txt"
+        AFTER_FILE="/tmp/${FILE_PREFIX}-after.txt"
+        # This is an update, so if enabled, compare the before/after resource lists
+        if [ ! -z "$OVERCLOUD_UPDATE_CHECK" ]; then
+            heat resource-list -n5 overcloud | awk '{print $2, $4, $6}' | sort > $BEFORE_FILE
+        fi
+
+        log "Overcloud update started."
+        openstack overcloud deploy $OVERCLOUD_UPDATE_ARGS
+        log "Overcloud update - DONE."
+
+        if [ ! -z "$OVERCLOUD_UPDATE_CHECK" ]; then
+            heat resource-list -n5 overcloud | awk '{print $2, $4, $6}' | sort > $AFTER_FILE
+            diff_rsrc=$(diff $BEFORE_FILE $AFTER_FILE)
+            if [ ! -z "$diff_rsrc" ]; then
+                log "Overcloud update - Completed but unexpected resource differences: $diff_rsrc"
+                exit 1
+            fi
+        fi
+        log "Overcloud update - DONE."
+        if [[ $OVERCLOUD_UPDATE_RM_FILES == 1 ]]; then
+            rm -f $BEFORE_FILE $AFTER_FILE
+        fi
+    else
+        log "Overcloud FAILED - No stack $OVERCLOUD_NAME."
+        exit 1
+    fi
 }
 
 function overcloud_delete {
@@ -493,13 +525,14 @@ function overcloud_pingtest {
 
     overcloudrc_check
 
-    IMAGE_PATH=$OVERCLOUD_IMAGES_PATH/fedora-user.qcow2
-    if [ ! -e $IMAGE_PATH ]; then
-        log "Overcloud pingtest, trying to download Fedora image"
-        curl -L -o $IMAGE_PATH http://cloud.fedoraproject.org/fedora-21.x86_64.qcow2
+    version=`curl http://download.cirros-cloud.net/version/released`
+    cirros_latest=cirros-$version-x86_64-disk.img
+    if [ ! -e $OVERCLOUD_IMAGES_PATH/$cirros_latest ]; then
+        log "Overcloud pingtest, trying to download latest $cirros_latest"
+        curl -o $OVERCLOUD_IMAGES_PATH/$cirros_latest http://download.cirros-cloud.net/$version/$cirros_latest
     fi
     log "Overcloud pingtest, uploading demo tenant image to glance"
-    glance image-create --progress --name pingtest_image --is-public True --disk-format qcow2 --container-format bare --file $IMAGE_PATH
+    glance image-create --progress --name pingtest_image --is-public True --disk-format qcow2 --container-format bare --file $OVERCLOUD_IMAGES_PATH/$cirros_latest
 
     log "Overcloud pingtest, creating demo tenant keypair and external network"
     if ! nova keypair-show default 2>/dev/null; then tripleo user-config; fi
@@ -516,14 +549,18 @@ function overcloud_pingtest {
     fi
     log "Overcloud pingtest, creating tenant-stack heat stack:"
     heat stack-create -f $TENANT_PINGTEST_TEMPLATE tenant-stack || exitval=1
-    if tripleo wait_for -w 360 -d 10 -s "CREATE_COMPLETE" -- "heat stack-list | grep tenant-stack"; then
+    if tripleo wait_for -w 180 -d 10 -s "CREATE_COMPLETE" -- "heat stack-list | grep tenant-stack"; then
         log "Overcloud pingtest, heat stack CREATE_COMPLETE";
 
-        vm1_ip=`heat output-show tenant-stack server1_public_ip -F raw`
+        if [ "$OVERCLOUD_PINGTEST_OLD_HEATCLIENT" = 1 ]; then
+            vm1_ip=`heat output-show tenant-stack server1_public_ip -F raw`
+        else
+            vm1_ip=`heat output-show tenant-stack server1_public_ip -F raw -v`
+        fi
 
         log "Overcloud pingtest, trying to ping the floating IPs $vm1_ip"
 
-        if tripleo wait_for -w 360 -d 10 -s "bytes from $vm1_ip" -- "ping -c 1 $vm1_ip" ; then
+        if tripleo wait_for -w 180 -d 10 -s "bytes from $vm1_ip" -- "ping -c 1 $vm1_ip" ; then
             ping -c 1 $vm1_ip
             log "Overcloud pingtest, SUCCESS"
         else
@@ -579,14 +616,14 @@ if [ "$INTROSPECT_NODES" = 1 ]; then
     introspect_nodes
 fi
 
-if [ "$FLAVORS" = 1 ]; then
-    flavors
-fi
-
 if [ "$OVERCLOUD_DEPLOY" = 1 ]; then
     overcloud_deploy
 fi
 
+if [ "$OVERCLOUD_UPDATE" = 1 ]; then
+    overcloud_update
+fi
+
 if [ "$OVERCLOUD_DELETE" = 1 ]; then
     overcloud_delete
 fi
@@ -606,6 +643,5 @@ if [ "$ALL" = 1 ]; then
     overcloud_images
     register_nodes
     introspect_nodes
-    flavors
     overcloud_deploy
 fi
